Package: fastei
Title: Methods for ''A Fast Alternative for the R x C Ecological
        Inference Case''
Version: 0.0.19
Authors@R: c(person(given = "Charles", family = "Thraves", role = "aut", email = "cthraves@dii.uchile.cl", comment = c(ORCID = "0000-0001-7491-7477")),
             person(given = "Pablo", family = "Ubilla", role = "aut", comment = c(ORCID = "0009-0003-9048-1767")),
             person(given = "Daniel", family = "Hermosilla", role = c("aut", "cre"), email = "daniel.hermosilla.r@ug.uchile.cl"))
Description: Estimates the probability matrix for the R×C Ecological Inference problem using the Expectation-Maximization Algorithm with four approximation methods for the E-Step, and an exact method as well. It also provides a bootstrap function to estimate the standard deviation of the estimated probabilities. In addition, it has functions that aggregate rows optimally to have more reliable estimates in cases of having few data points. For comparing the probability estimates of two groups, a Wald test routine is implemented. The library has data from the first round of the Chilean Presidential Election 2021 and can also generate synthetic election data. Methods described in Thraves, Charles; Ubilla, Pablo; Hermosilla, Daniel (2024) ''A Fast Ecological Inference Algorithm for the R×C case'' <doi:10.2139/ssrn.4832834>.
License: MIT + file LICENSE
Encoding: UTF-8
Imports: Rcpp, jsonlite
LinkingTo: Rcpp
NeedsCompilation: yes
BugReports: https://github.com/DanielHermosilla/ecological-inference-elections/issues
URL:
        https://danielhermosilla.github.io/ecological-inference-elections/reference/fastei-package.html
Suggests: knitr, rmarkdown, ggplot2, reshape2, viridis, dplyr, qpdf,
        testthat (>= 3.0.0)
VignetteBuilder: knitr
Depends: R (>= 3.5)
LazyData: true
RoxygenNote: 7.3.3
Config/testthat/edition: 3
Packaged: 2026-05-20 14:50:32 UTC; daniel
Author: Charles Thraves [aut] (ORCID: <https://orcid.org/0000-0001-7491-7477>),
  Pablo Ubilla [aut] (ORCID: <https://orcid.org/0009-0003-9048-1767>),
  Daniel Hermosilla [aut, cre]
Maintainer: Daniel Hermosilla <daniel.hermosilla.r@ug.uchile.cl>
Repository: CRAN
Date/Publication: 2026-05-20 16:40:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-05-22 00:14:35 UTC; unix
Archs: fastei.so.dSYM
