# Generated by roxygen2: do not edit by hand

S3method(as.matrix,eim)
S3method(logLik,eim)
S3method(plot,eim)
S3method(print,eim)
S3method(summary,eim)
export(bootstrap)
export(eim)
export(get_agg_opt)
export(get_agg_proxy)
export(get_eim_chile)
export(run_em)
export(save_eim)
export(simulate_election)
export(waldtest)
import(Rcpp)
importFrom(Rcpp,evalCpp)
importFrom(stats,logLik)
importFrom(stats,pnorm)
importFrom(stats,pt)
importFrom(stats,rgamma)
importFrom(stats,rmultinom)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(utils,modifyList)
importFrom(utils,tail)
importFrom(utils,write.csv)
useDynLib(fastei, .registration = TRUE)
